/*
 * Copyright 2015 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

package fr.ill.ics.cameo.examples;

import fr.ill.ics.cameo.Application;
import fr.ill.ics.cameo.RemoteException;
import fr.ill.ics.cameo.Server;


public class RequesterApp {

	public static void main(String[] args) {
		
		Application.This.init(args);
		
		String requestMessage = "";
		int N = 1;
		Server server = null;
		
		// The request message is the first argument.
		if (args.length > 0) {
			requestMessage = args[0];
		}
		
		// The number of requests is the second argument.
		if (args.length > 1) {
			N = Integer.parseInt(args[1]);
		}
		
		// The server endpoint is the third argument.
		if (args.length > 2) {
			server = new Server(args[2]);
		}
		else {	
			server = new Server(Application.This.getEndpoint());
		}
		
		if (Application.This.isAvailable() && server.isAvailable()) {
			System.out.println("Connected server " + server);
		} else {
			System.exit(-1);
		}
		
		try {
			// Connect to the server.
			Application.Instance responderServer = server.connect("responder");
			System.out.println("Application " + responderServer + " has state " + Application.State.toString(responderServer.getActualState()));
			
			// Create a requester.
			Application.Requester requester = Application.Requester.create(responderServer, "the-responder");
			System.out.println("Created requester " + requester);
			
			for (int i = 0; i < N; ++i) {
				// Send a simple message as string.
				requester.send(requestMessage + "-" + i);
				System.out.println("Response is " + requester.receiveString());
			}
				
			// Terminate the requester and server.
			requester.terminate();
			server.terminate();
			
		} catch (RemoteException e) {
			System.out.println("Requester error:" + e);
			
		} finally {
			// Do not forget to terminate This.
			Application.This.terminate();
		}
		
		System.out.println("Finished the application");
	}

}